const https = require("https");
const http = require("http");
const path = require("path");
const fs = require("fs");
const bytesToSize = require("./bytesToSize");
const { unlink, mkdir } = require("fs").promises;

class Download {
  constructor() {
    this.totalBytes = 0;
    this.downloadedBytes = 0;
    this.lastDownloaded = 0;
  }
  download = () => {
    return new Promise((resolve, reject) => {
      const file = fs.createWriteStream(this.tempFile);
      this.request = this.http.get(this.url, (response) => {
        if (response.statusCode !== 200 && response.statusCode !== 206) {
          reject(new Error("文件下载失败,错误码：" + response.statusCode));
          return;
        }
        if (!this.totalBytes) {
          this.totalBytes = parseInt(response.headers["content-length"], 10);
        }
        response.pipe(file);
        response.on("data", (chunk) => {
          this.downloadedBytes += chunk.length;
        });

        file.on("finish", () => {
          file.close();
          fs.rename(this.tempFile, this.dest, (err) => {
            if (err) throw err;
            resolve();
          });
        });
        file.on("error", function (err) {
          fs.unlink(this.tempFile);
          reject(err);
        });
        this.request.on("error", function (err) {
          reject(err);
        });
      });
    });
  };
  start = async (url, dir, callback = () => {}) => {
    const { protocol, pathname } = new URL(url);
    this.url = url;
    const fileName = pathname.substring(pathname.lastIndexOf("/") + 1);
    // 真实的文件路径
    this.dest = path.join(dir, fileName);
    if (fs.existsSync(this.dest)) {
      return this.dest;
    }
    await mkdir(dir, { recursive: true });
    this.tempFile = this.dest + ".download";
    if (fs.existsSync(this.tempFile)) {
      await unlink(this.tempFile);
    }
    if (protocol === "https:") {
      this.http = https;
    } else if (protocol === "http:") {
      this.http = http;
    } else {
      throw new Error("valid protocol. only support https and http.");
    }
    this.interval = setInterval(() => {
      const res = {
        completedSize: bytesToSize(this.downloadedBytes),
        totalSize: bytesToSize(this.totalBytes),
        downloadSpeed: bytesToSize(this.downloadedBytes - this.lastDownloaded),
        percent: this.totalBytes === 0 ? 0 : ((this.downloadedBytes / this.totalBytes) * 100).toFixed(2),
      };
      this.lastDownloaded = this.downloadedBytes;
      callback(res);
    }, 1000);
    try {
      await this.download();
      clearInterval(this.interval);
    } catch (e) {
      clearInterval(this.interval);
      throw e;
    }
    return this.dest;
  };
  close = () => {
    clearInterval(this.interval);
    this.request.destroy();
  };
}

module.exports = (url, dir, callback, ref) => {
  const download = new Download();
  if (ref) ref(download);
  return download.start(url, dir, callback);
};
