const conda = require("./conda");
const path = require("path");
const fs = require("fs");
const aria2c = require("../utils/aria2c");
const getAvailablePort = require("../utils/getAvailablePort");
const killChild = require("../utils/killChild");
const is = require("electron-is");
const download = require("../utils/download");
class LamaCleaner {
  constructor() {
    const appPath = path.join(__dirname, "../../../");
    this.modelPath = path.join(appPath, "/lib/torch-home/hub/checkpoints/");

    this.bigLamaPath = path.join(this.modelPath, "/big-lama.pt");

    process.env.PIP_INDEX_URL = "https://mirrors.aliyun.com/pypi/simple/";
  }
  checkDownload = () => {
    if (!fs.existsSync(this.bigLamaPath) || fs.existsSync(this.bigLamaPath + ".aria2")) {
      return false;
    }
    return true;
  };
  downloadModel = async (callback) => {
    if (!this.checkDownload()) {
      const data = { id: "big_lama", type: "download", name: "模型" };
      callback(data);
      try {
        await aria2c.download("https://ghproxy.com/https://github.com/Sanster/models/releases/download/add_big_lama/big-lama.pt", { dir: this.modelPath }, (res) => {
          data.data = res;
          callback(data);
        });
      } catch (e) {
        callback({ globalError: "启动多线程下载器aria2失败，请给予权限后重试" });
        throw new Error("下载失败");
      }
      data.finish = true;
      callback(data);
      this.downloadFinish = true;
      this.startLamaCleaner(callback);
    }
  };
  checkLamaCleaner = async (callback) => {
    const data = { id: "installMiniconda", name: "环境检查" };
    callback && callback(data);
    let res;
    try {
      await conda.exec("pip show lama-cleaner");
      res = true;
      this.installFinish = true;
    } catch (e) {
      this.installFinish = false;
      res = false;
    }
    data.finish = true;
    callback && callback(data);
    return res;
  };
  installLamaCleaner = async (callback) => {
    const data = { id: "installLamaCleaner", name: "LamaCleaner" };
    callback(data);
    if (await this.checkLamaCleaner()) {
      data.finish = true;
      callback(data);
      return;
    }
    await conda.spawn("pip install lama-cleaner", { log: true });
    await this.installLamaCleaner(callback);
  };

  runningCallback = (callback) => {
    this.running = true;
    const data = { id: "running", running: true, port: this.port };
    callback(data);
  };
  startLamaCleaner = async (callback) => {
    if (!this.installFinish || !this.downloadFinish) return;

    const data = { id: "start", name: "程序启动中", type: "start" };
    callback(data);
    this.port = await getAvailablePort(14874);
    const device = is.windows() ? "cuda" : "cpu";
    const command = `lama-cleaner --model=lama --device=${device} --port=${this.port} --disable-model-switch`;
    await conda.spawn(command, {
      log: true,
      stderr: (data) => {
        if (data.indexOf("Running on") >= 0) {
          this.runningCallback(callback);
        }
      },
      getChild: (child) => (this.child = child),
    });
    this.running = false;
  };
  finish = (callback) => {
    if (this.installFinish && this.downloadFinish) {
      this.start(callback);
    }
  };
  install = async (callback) => {
    await conda.install(callback);
    await conda.installPytorch(callback);
    await this.installLamaCleaner(callback);
    // 安卓完成
    this.startLamaCleaner(callback);
  };
  close = async () => {
    await conda.close();
    await aria2c.close();
    killChild(this.child);
  };
  start = async (callback) => {
    if (this.running) {
      this.runningCallback(callback);
      return;
    }
    if (this.checkDownload()) {
      this.downloadFinish = true;
    } else {
      this.downloadModel(callback);
    }
    if (await this.checkLamaCleaner(callback)) {
      this.startLamaCleaner(callback);
    } else {
      this.install(callback);
    }
  };
}

const lamaCleaner = new LamaCleaner();
module.exports = lamaCleaner;
